# @design.estate/dees-comms
a comms module for communicating within the DOM

## Install
To integrate `@design.estate/dees-comms` into your project, run the following command in your terminal:

```bash
npm install @design.estate/dees-comms --save
```

This will add the package to your project dependencies. 

## Usage

`@design.estate/dees-comms` facilitates communication within the DOM, especially between various workers and tabs in client-side JavaScript. Using TypeScript and the provided ESM syntax will enhance your development experience with better intellisense support and type safety.

### Setting Up

First, make sure to import the module in your TypeScript file:

```typescript
import { DeesComms } from '@design.estate/dees-comms';
```

### Initializing the Communication Module

Create an instance of the `DeesComms` class to start using the module. This instance will act as the central communication hub in your application.

```typescript
const myDeesComms = new DeesComms();
```

### Sending Messages

To send messages, you'll need to create a typed request. Let's assume we are sending a message that requires a response:

```typescript
// Define the message type
interface MyMessageType {
  method: 'myMethod';
  someData: string;
}

// Creating a typed request
const myTypedRequest = myDeesComms.createTypedRequest<MyMessageType>('myMethod');

// Firing the request
myTypedRequest.fire({
  someData: 'Hello, World!',
}).then(response => {
  console.log('Received response:', response);
});
```

### Receiving and Handling Messages

To handle incoming messages, you need to define handlers for the types of messages you expect to receive. 

```typescript
// Define the handler for the 'myMethod' message
myDeesComms.createTypedHandler<MyMessageType>('myMethod', async (incomingMessage) => {
  console.log('Message received:', incomingMessage);
  
  // Process the incoming message...
  
  // Return a response (if needed)
  return {
    response: 'Message received loud and clear!'
  };
});
```

### Communication Between Tabs or Workers

`@design.estate/dees-comms` uses the Broadcast Channel API and a polyfill for environments where it's not available, enabling seamless communication between different contexts like tabs, iframes, or web workers.

### Clean Up

When you're done with communication, or before creating a new instance for any reason, ensure you clean up properly to prevent memory leaks or other unexpected behaviors:

```typescript
// Currently, the cleanup should be handled according to the specific needs of your application
// e.g., closing or nullifying references to the DeesComms instance.
```

### Complete Example

Putting it all together, let's create a small application that sends a message and listens for a response:

```typescript
import { DeesComms } from '@design.estate/dees-comms';

const appDeesComms = new DeesComms();

// Setting up the listener
appDeesComms.createTypedHandler<{ method: 'greet'; name: string }>('greet', async (message) => {
  console.log(`Greetings received from ${message.name}`);
  return { reply: `Hello ${message.name}, nice to meet you!` };
});

// Sending a greeting message
const greetRequest = appDeesComms.createTypedRequest<{ method: 'greet'; name: string }>('greet');
greetRequest.fire({ name: 'Alice' }).then(response => {
  console.log(response.reply); // Expected output: "Hello Alice, nice to meet you!"
});
```

This module provides a powerful and flexible way to handle communications within the DOM, leveraging modern JavaScript features and patterns. The usage of TypeScript ensures type safety and enhances code understandability and maintainability.

## License and Legal Information

This repository contains open-source code that is licensed under the MIT License. A copy of the MIT License can be found in the [license](license) file within this repository. 

**Please note:** The MIT License does not grant permission to use the trade names, trademarks, service marks, or product names of the project, except as required for reasonable and customary use in describing the origin of the work and reproducing the content of the NOTICE file.

### Trademarks

This project is owned and maintained by Task Venture Capital GmbH. The names and logos associated with Task Venture Capital GmbH and any related products or services are trademarks of Task Venture Capital GmbH and are not included within the scope of the MIT license granted herein. Use of these trademarks must comply with Task Venture Capital GmbH's Trademark Guidelines, and any usage must be approved in writing by Task Venture Capital GmbH.

### Company Information

Task Venture Capital GmbH  
Registered at District court Bremen HRB 35230 HB, Germany

For any legal inquiries or if you require further information, please contact us via email at hello@task.vc.

By using this repository, you acknowledge that you have read this section, agree to comply with its terms, and understand that the licensing of the code does not imply endorsement by Task Venture Capital GmbH of any derivative works.
